package com.wanji.indicators.task.track.service;

import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.RoutePathCarTrackModel;
import com.wanji.indicators.task.track.service.func.SplitRoutePathFunction;
import com.wanji.indicators.task.track.service.key.KeySelectorGlobalIdPlateNo;
import com.wanji.indicators.task.track.service.sink.PartitionRoutePathElasticSearchSinkFunction;
import com.wanji.indicators.task.track.service.sink.RoutePathElasticSearchSinkFunction;
import com.wanji.indicators.util.ElasticSearchSinkUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.http.HttpHost;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.time.Duration;
import java.util.List;
import java.util.Properties;

public class CarRoutePathMainNew   {
    private final StreamExecutionEnvironment env;

    private final PropertiesHelper instance;

    private Properties p = new Properties();

    private String path;

    private String indexName;


    private CarRoutePathMainNew(StreamExecutionEnvironment env, String indexName) {
        this.env = env;
        this.indexName = indexName;
        this.instance = PropertiesHelper.getInstance();
        this.p = this.instance.getProperties();
    }


    public static CarRoutePathMainNew init(StreamExecutionEnvironment env, String indexName) {
        return new CarRoutePathMainNew(env, indexName);
    }


    public void run(SingleOutputStreamOperator<CarTrackModel> carTrackModelStream) {

        //给每个车辆轨迹点进行路径规划分段,连续两点超过半小时的重新生成一条新路线
        SingleOutputStreamOperator<RoutePathCarTrackModel> splitLineStream = carTrackModelStream
                .assignTimestampsAndWatermarks(
                        WatermarkStrategy.<CarTrackModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
                .keyBy(new KeySelectorGlobalIdPlateNo())
                .process(new SplitRoutePathFunction())
                .setParallelism(2).name("提取车辆通行路径");

        //从配置文件中读取 es 的地址
        try {

            List<HttpHost> esAddresses = ElasticSearchSinkUtil.getEsAddresses(p.getProperty("elasticsearch.server"));

            RoutePathElasticSearchSinkFunction elasticsearchSinkFunction = new RoutePathElasticSearchSinkFunction(indexName);
            ElasticSearchSinkUtil.addSink(esAddresses, 2, splitLineStream, elasticsearchSinkFunction);

            PartitionRoutePathElasticSearchSinkFunction elasticsearchSinkFunction1 = new PartitionRoutePathElasticSearchSinkFunction(indexName);
            ElasticSearchSinkUtil.addSink(esAddresses, 2, splitLineStream, elasticsearchSinkFunction1);

        } catch (MalformedURLException e) {
            e.printStackTrace();
        }

    }


}
